"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PushFileSystem = void 0;
/* eslint
    no-console:"off",
*/
var fs = __importStar(require("fs"));
var glob_1 = __importDefault(require("glob"));
var path = __importStar(require("path"));
var PushFileSystem = /** @class */ (function () {
    function PushFileSystem(io, datadir) {
        this.io = io;
        this.datadir = datadir;
        this.resolvedDataDir = path.resolve(this.datadir);
        this.datadirglob = this.resolvedDataDir + "/**";
    }
    PushFileSystem.prototype.getNodes = function (callback) {
        var _this = this;
        (0, glob_1.default)(this.datadirglob, { mark: true }, function (err, rawNodes) {
            if (err) {
                console.error(err);
                return;
            }
            console.log("raw nodes");
            console.log(rawNodes);
            if (rawNodes.length === 0) {
                console.error("no items in directory");
                callback([]);
            }
            else {
                var first_1 = rawNodes[0];
                if (path.relative(_this.datadir, first_1) !== "") {
                    console.log(first_1);
                    console.log(_this.resolvedDataDir);
                    throw new Error("expected the first entry to be the root");
                }
                callback(rawNodes
                    .slice(1, rawNodes.length)
                    .map(function (rm) {
                    if (!rm.startsWith(first_1)) {
                        throw new Error("entry does not start with expected root");
                    }
                    return rm.substring(first_1.length);
                })
                    .sort(function (a, b) {
                    return a.localeCompare(b);
                }));
            }
        });
    };
    PushFileSystem.prototype.reportFile = function (change, node) {
        this.io.emit(change, JSON.stringify(this.parsePath(node)));
        console.error('file:', change, node, this.parsePath(node));
    };
    PushFileSystem.prototype.parsePath = function (nodePath) {
        var parsed = path.parse(nodePath);
        return {
            dir: parsed.dir,
            base: parsed.base,
            type: nodePath.endsWith("/")
                ? ["dir", {
                        globalID: nodePath.substr(0, nodePath.length - 1), //strip the trailing slash
                    }]
                : ["file"],
        };
    };
    PushFileSystem.prototype.pushFileSystem = function () {
        var _this = this;
        this.getNodes(function (nodes) {
            var currentNodes = nodes;
            console.log("current nodes");
            console.log(currentNodes);
            _this.io.onConnection(function (socket) {
                socket.emit("initial-directory-structure", JSON.stringify(currentNodes.map(function (cm) { return _this.parsePath(cm); })));
            });
            fs.watch(_this.resolvedDataDir, { persistent: true, recursive: true }, function () {
                _this.getNodes(function (newNodes) {
                    var i = 0;
                    var j = 0;
                    while (true) {
                        if (i === currentNodes.length && j === newNodes.length) {
                            break;
                        }
                        if (i === currentNodes.length) {
                            //no more current nodes
                            _this.reportFile('added', newNodes[j]);
                            j++;
                        }
                        else if (j === newNodes.length) {
                            //no more new nodes
                            _this.reportFile('removed', currentNodes[i]);
                            i++;
                        }
                        else {
                            if (newNodes[j] < currentNodes[i]) {
                                _this.reportFile('added', newNodes[j]);
                                j++;
                            }
                            else if (newNodes[j] > currentNodes[i]) {
                                _this.reportFile('removed', currentNodes[i]);
                                i++;
                            }
                            else {
                                i++;
                                j++;
                            }
                        }
                    }
                    currentNodes = newNodes;
                });
            });
        });
    };
    return PushFileSystem;
}());
exports.PushFileSystem = PushFileSystem;
