"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.startWebserver = void 0;
/* eslint
    no-console:"off",
*/
var express_1 = __importDefault(require("express"));
var fs = __importStar(require("fs"));
var http = __importStar(require("http"));
var url = __importStar(require("url"));
var path = __importStar(require("path"));
var p = __importStar(require("pareto"));
var socket_io_1 = __importDefault(require("socket.io"));
var makeNativeHTTPrequest_1 = require("./makeNativeHTTPrequest");
var fileSystem_1 = require("./fileSystem");
function directoryExists(dirPath) {
    return fs.existsSync(dirPath) && fs.lstatSync(dirPath).isDirectory();
}
function startWebserver(port) {
    var app = (0, express_1.default)();
    var httpServer = http.createServer(app);
    var io = new socket_io_1.default.Server(httpServer);
    var _a = process.argv, node = _a[0], script = _a[1], staticdir = _a[2], datadir = _a[3];
    if (staticdir === undefined) {
        console.error("format: ".concat(node, " ").concat(script, ", <staticdir> <datadir>"));
        process.exit(1);
    }
    if (!directoryExists(staticdir)) {
        console.error("static directory ".concat(staticdir, " does not exist"));
        process.exit(1);
    }
    if (datadir === undefined) {
        console.error("format: ".concat(node, " ").concat(script, ", <staticdir> <datadir>"));
        process.exit(1);
    }
    if (!directoryExists(datadir)) {
        console.error("data directory ".concat(datadir, " does not exist"));
        process.exit(1);
    }
    app.get("/", function (_req, res) {
        res.redirect("/static/index.html");
    });
    app.get("/index.html", function (_req, res) {
        res.redirect("/static/index.html");
    });
    app.get("/favicon.ico", function (_req, res) {
        res.redirect("/static/favicon.ico");
    });
    function addAppGet(prefix, callback) {
        var prefixWithSlashes = "/".concat(prefix, "/");
        app.get("".concat(prefixWithSlashes, "*"), function (req, res) {
            var normalizedURL = path.normalize(req.url.substr("".concat(prefixWithSlashes).length));
            callback(res, normalizedURL);
        });
    }
    function addDir(prefix, dir) {
        addAppGet(prefix, function (res, normalizedURL) {
            if (normalizedURL.startsWith("..")) {
                res.statusCode = 403;
                res.sendFile(path.resolve('notAllowed.html'));
            }
            else {
                res.sendFile(path.resolve(path.join(dir, decodeURIComponent(normalizedURL))));
            }
        });
    }
    addDir("data", path.resolve(datadir));
    addDir("static", path.resolve(staticdir));
    addAppGet("http", function (res, normalizedURL) {
        var parsedURL = url.parse("http://" + normalizedURL);
        console.log(normalizedURL);
        console.log(parsedURL.host, parsedURL.path);
        if (parsedURL.host === null || parsedURL.path === null) {
            throw new Error("missing http host and/or path");
        }
        (0, makeNativeHTTPrequest_1.makeNativeHTTPrequest)({
            host: parsedURL.host,
            path: parsedURL.path,
            timeout: 2000,
        }).handle(function (_error) {
            res.statusCode = 404;
            res.sendFile(path.resolve('urlNotFound.html'));
        }, function (stream) {
            var allData = "";
            stream.handle(null, {
                onData: function (data) {
                    allData += data;
                    return p.value(false);
                },
                onEnd: function () {
                    console.log("SENDING");
                    res.send(allData);
                },
            });
        });
    });
    httpServer.listen(port, function () {
        console.log("listening on *: ".concat(port));
    });
    var ioWrapper = {
        emit: function (event, data) {
            io.emit(event, data);
        },
        onConnection: function (callback) {
            io.on("connection", function (s) {
                callback(s);
            });
        },
    };
    new fileSystem_1.HandleCommands(ioWrapper, datadir).handleCommands();
    new fileSystem_1.PushFileSystem(ioWrapper, datadir).pushFileSystem();
}
exports.startWebserver = startWebserver;
